/** 
*  \file CalibrationSequencer.hxx
*  \brief Simple controlling state machines for pushing data through calibration pipelines.
*
*  \author R.K.Garcia <rayg@ssec.wisc.edu>
*
*  \version $Id: CalibrationSequencer.hxx,v 1.2 2005/11/09 16:12:04 rayg Exp $
*
*  \par Copyright:
*  \verbatim
*
*  Copyright UW/SSEC, ALL RIGHTS RESERVED, 2004
*  Space Science and Engineering Center
*  University of Wisconsin - Madison, USA
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*  \endverbatim
*/

#ifndef HXX_CAL_SEQ
#define HXX_CAL_SEQ

#include <list>
#include <queue>

/* Interface specification

BracketedCalibrationSequencer<Traits,Ports>
    // we assume that the input sequence is punctuated by blocks of blackbody views
    // regex form: ([HAZ]+E+)+
    // the [HAZ] portions are routed to a study pipeline
    // once a given HAZ block has terminated, the E's preceding it can be calibrated.
    
    // the pipelines are assumed to require monotonic time on their inputs.
    calstudy_ctl->operator() : invocation for study pipeline
    calapply_ctl->operator() : invocation for calibration application pipeline
    
    // obs_in represents the record seen by the pipelines.
    // obs_in is a record store which is bidirectional iterator-like.
    // it iterates through the input records available.
    // it is copyable and assignable.
    ++obs_in : next record
    --obs_in : last record
    obs_in += N : jump by N records (positive or negative)
    obs_in -= N : jump backwards by N records (N can be positive or negative)
    !obs_in : indicates end of input

    iterator : type of obs_in    
    // content: 
    obs_in->time
    obs_in->scenetype
    obs_in->scandir


BatchStudyCalibrationSequencer<Traits,Ports>
// only processes HAZ's, ignores E's, only increments obs_in

BatchApplyCalibrationSequencer<Traits,Ports>
// only processes E's, ignores HAZ's, only increments obs_in

PredictiveCalibrationSequencer<Traits,Ports>
    // only increments obs_in
    // assumes that calapply pipeline can do useful prediction
    
PredictiveCalibrationSequencerWithErrorAnalysis<Traits,Ports>
    // combines predictive with bracketing: 
    // present predictive calibration; as brackets close, allow a third pipeline to do 
    // error analysis on the results of the predictive
    // requires bidirectional iterator behavior on obs_in
        
*/

#ifndef DBG
#define DBG(x) x
#endif



template< typename Traits, typename Ports >
class BracketedCalibrationSequencer
{
protected:
    Ports P;
    enum { AWAIT_1ST_EARTH , BUFFERING
    
    typedef std::queue< Ports::iterator, std::list<Ports::iterator> > PendingQueueType;
    
    PendingQueueType pending;
    
public:

    BracketedCalibrationSequencer( const Ports &p ): P(p) { }
    
    int operator()( void )
    {
        for(;;)
        {
            if (!P.obs_in) break;
            
            switch( P.obs_in->scenetype )
            {
                case 'E': 
            }
            
        }
        return 0;
    }

};


#endif
