/**
 * \file Ifg2Spectrum.h
 * \brief declarations (not definitions) for the Interferogram to Spectrum abstract stage
 *
 * \author Maciek Smuga-Otto <maciek@ssec.wisc.edu>
 *
 * based directly on the Scanning-HIS rsh2fbf application, written by Ray Garcia <rayg@ssec.wisc.edu>
 *
 *  \version $Id: Cifg2Spectrum.hxx,v 1.4 2005/10/28 02:08:32 rayg Exp $
 *
 *  \par Copyright:
 *  \verbatim
 *
 *  Copyright UW/SSEC, ALL RIGHTS RESERVED, 2004
 *  Space Science and Engineering Center
 *  University of Wisconsin - Madison, USA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *  \endverbatim
 *
 */

/**
 * \mainpage Converting Interferograms to Spectra
 *
 * \image html overview.jpg
 *
 * \image html capsule_diagram.jpg
 *
 */

#ifndef H_IFG2SPECTRUM
#define H_IFG2SPECTRUM

#ifndef NO_DEPENDENCY_INCLUDES
#include <complex>

// GIPS includes
#include "Physics.h"
#include "Instrument.h"
#include "ProtocolRole.h"
#include "Operator.h"

#include "Types.h"

#include <boost/shared_ptr.hpp>

#endif

// NOTE: in the future, this file will be auto-generated by a flanging compiler.

namespace gips {


template< typename Traits >
class Cifg2SpectrumInterface
{
public:
    typedef BufferTypes<Traits> Types;
    
    struct settings_t 
    {
        unsigned int chopRawSpectrumFrom; 
        unsigned int chopRawSpectrumTo;
        char wrapRawSpectrumLastPoint;
    };
    
    struct obs_in_t
    {
        typename Types::RawInterferogram interferogram;
    };
    
    struct obs_out_t
    {
        typename Types::Spectrum cplxspectrum;
    };
    
    struct Ports 
    {
        typedef const settings_t *settings_p;
        settings_p settings;
        
        typedef const obs_in_t *obs_in_p;
        obs_in_p obs_in;
        
        typedef obs_out_t *obs_out_p;
        obs_out_p obs_out;
            
        Ports(settings_p settings_,
              obs_in_p obs_in_,
              obs_out_p obs_out_):
           settings(settings_), obs_in(obs_in_), obs_out(obs_out_)
        {
        }
    };
};


// FIXME: decide if Cifg2Spectrum inherits from traits - just simplifies syntax on use of typename etc.
template<typename Traits, typename Ports>
class Cifg2Spectrum 
{
protected:
    typedef BufferTypes<Traits> Types;
    Ports P;
    
    // constants local to this capsule, declared in terms of the protocol traits
    enum { rawInterferogramSize = Types::RawInterferogramSize, 
           shiftedInterferogramSize = Types::InterferogramSize, 
           doubledRawSpectrumSize = Types::InterferogramSize, 
           rawSpectrumSize = Types::SpectrumSize };
    typedef typename Types::SpectrumBuffer<doubledRawSpectrumSize> DoubledRawSpectrum;
       
public:
    virtual Operator::Result operator()( void ) ///< run this stage on input data provided over the ports.
    { 
        return Operator::UNIMPLEMENTED; 
    }

    virtual ~Cifg2Spectrum() { } 	    

    explicit Cifg2Spectrum(const Ports &p): 
             P(p) 
    {
    } 

}; // class Ifg2Spectrum
    
} // namespace
#endif

