/*
 * \brief SHIS implementation of Ifg2SpectrumStage
 * \author Maciek Smuga-Otto <maciek@ssec.wisc.edu>
 *
 * based directly on the Scanning-HIS rsh2fbf application, written by Ray Garcia <rayg@ssec.wisc.edu>
 *
 *  \version $Id: Cifg2Spectrum_GIFTSv1.hxx,v 1.6 2005/11/22 03:21:43 maciek Exp $
 *
 *  \par Copyright:
 *  \verbatim
 *
 *  Copyright UW/SSEC, ALL RIGHTS RESERVED, 2004
 *  Space Science and Engineering Center
 *  University of Wisconsin - Madison, USA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *  \endverbatim
 */

// NOTE: THIS FILE WILL BE AUTO-GENERATED IN THE FUTURE FROM INTEFACE DESCRIPTIONS
// OF THE SUBCAPSULES BY MEANS OF A FLANGING COMPILER

#include <complex>
#include <vector>

#include <boost/shared_ptr.hpp>

#include "IfgShift.hxx"
#include "CplxFFT.hxx"
#include "SpectrumChop.hxx"

#include "Cifg2Spectrum.hxx"

namespace gips {

// Ci2sImpl is an acronym for (C)omplex (i)nterferogram (to) (s)pectrum (Impl)emenatation
template<typename Traits, typename Ports>
class Ci2sImpl: public Cifg2Spectrum<Traits,Ports>
{
protected:

    // ease-of-use typedefs
    typedef Cifg2Spectrum<Traits,Ports> Super;

    // typedef BufferTypes<Traits> Types; is in superclass    
    // Ports P; is in superclass

    ////////////
    // define internal protocols and connections for the sub-capsules we're composing
    
    // IfgShift takes the buffer in from our input port and outputs to an internal connector    
    struct IS_to_CF
    {
        typename Types::Interferogram cplxinterferogram;
    } is2fft;

    struct IS_Ports
    {
        typename Ports::obs_in_p obs_in;  // connector types, copyable
        IS_to_CF* obs_out;
    };
        
    // and a routine to initialize it
    IS_Ports _isPorts()
    {
        IS_Ports ret;
        ret.obs_in = P.obs_in;
        ret.obs_out = &is2fft;
        return ret;
    };
    
    typedef RealToCplxIfgShift<Traits,IS_Ports> MyIfgShift;

    //////////        
    // CplxFFT takes output of IfgShift and writes it to another intermediate buffer
     struct CF_to_SC
    {
        typename Types::FftSpectrum cplxspectrum;
    } fft2sc;

    struct CF_Ports
    {
        const IS_to_CF* obs_in;
        CF_to_SC* obs_out;
    };    
    
    CF_Ports _cfPorts()
    {
        CF_Ports ret;
        ret.obs_in = &is2fft;
        ret.obs_out = &fft2sc;
        return ret;
    }
    
    typedef CplxFFT<Traits,CF_Ports> MyCplxFFT;
    
    //////////        
    // finally, connect the SpectrumChop to the outgoing buffer
    
    // SpectrumChop internal protocols are memory-based using pointers as connectors
    // experimental: use the settings on our input port on the assumption it's field-compatible with the spectrumfold capsule
    
    struct SC_Ports
    {
        typename Ports::settings_p settings;
        const CF_to_SC* obs_in;
        typename Ports::obs_out_p obs_out;
    };
    
    SC_Ports _scPorts()
    {
        SC_Ports ret;
        ret.settings = P.settings;
        ret.obs_in = &fft2sc;
        ret.obs_out = P.obs_out;
        return ret;
    }

    typedef SpectrumChop<Traits,SC_Ports> MySpectrumChop;

    //////////

    MyIfgShift capsule1;
    MyCplxFFT capsule2;
    MySpectrumChop capsule3;
    
public:
    explicit Ci2sImpl( const Ports &ports ): Super( ports ),
        capsule1(_isPorts()), 
        capsule2(_cfPorts()), 
        capsule3(_scPorts())
    {
    }  
    
    ~Ci2sImpl() {};
    
    Operator::Result operator() ( )
    {   
        Operator::Result res1, res2, res3;
    
        //std::cerr << "C++: in Cifg2Spectrum::operator()" << std::endl; // DEBUG
        res1 = capsule1();
        //std::cerr << "buffer_1@600: " << buffer_1.shiftedInterferogram[600] << std::endl; // DEBUG
        res2 = capsule2();
        //std::cerr << "buffer_2@600: " << buffer_2.doubledRawSpectrum[600] << std::endl; // DEBUG
        res3 = capsule3();
        //std::cerr << "Cifg2Spectrum:" << res1 << res2 << res3 << std::endl; // DEBUG
        return (res1 == Operator::OK && res2 == Operator::OK && res3 == Operator::OK) 
            ? Operator::OK 
            : Operator::ERROR;
    }
}; // class
    
} // Namespace
